<?php
/*
 * @package QuantumGateway payment module
 * @copyright Copyright (c) 2007 Softcraft Impresa, LLC
 * @license GNU Public License V2.0 (http://www.zen-cart.com/license/2_0.txt)
 * @version $Id: quantumgateway.php,v 1.0.3 2007/05/30 softcraft Exp $
 *
 * Inspired by the Zen Cart cc payment method class
 * includes/modules/payment/cc.php 4903 2006-11-10 10:26:06Z drbyte
 * and quantumgateway.php,v 1.40 2006/05/10 18:23:14 dgw_
 * osCommerce (Open Source E-Commerce Solutions) by Jerry Brown
*/


// Quantum Gateway Transparent Database Engine agent.
define(MODULE_PAYMENT_QUANTUMGATEWAY_AGENT,
	'https://secure.quantumgateway.com/cgi/tqgwdbe.php');

// Code resonsible to dispatch HTTP POST request to the agent.
define(MODULE_PAYMENT_QUANTUMGATEWAY_DISPATCH,
	HTTPS_SERVER . DIR_WS_HTTPS_CATALOG . 'quantumgateway.php');


  class quantumgateway extends base {
    var $code, $title, $description, $enabled;

    /*
     * class constructor
     */
    function quantumgateway () {
      $this->code = 'quantumgateway';
      $this->title = MODULE_PAYMENT_QUANTUMGATEWAY_TEXT_TITLE;
      $this->description = MODULE_PAYMENT_QUANTUMGATEWAY_TEXT_DESCRIPTION;
      $this->sort_order = MODULE_PAYMENT_QUANTUMGATEWAY_SORT_ORDER;
      $this->enabled = ((MODULE_PAYMENT_QUANTUMGATEWAY_STATUS == 'True') ? true : false);

      if ((int)MODULE_PAYMENT_QUANTUMGATEWAY_ORDER_STATUS_ID > 0) {
	$this->order_status = MODULE_PAYMENT_QUANTUMGATEWAY_ORDER_STATUS_ID;
      }

      $this->form_action_url = MODULE_PAYMENT_QUANTUMGATEWAY_DISPATCH;
    } // </constructor>


    /*
     * javascript_validation - does error checking of data entry
     */
    function javascript_validation() {
      $js = '  if (payment_value == "' . $this->code . '") {' . "\n" .
            '    var cc_owner = document.checkout_payment.cc_owner.value;' . "\n" .
            '    var cc_number = document.checkout_payment.cc_number.value;' . "\n";

      if (MODULE_PAYMENT_QUANTUMGATEWAY_COLLECT_CVV == 'True')  {
        $js .= '    var cc_cvv = document.checkout_payment.cc_cvv.value;' . "\n";
    }

      $js .= '    if (cc_owner == "" || cc_owner.length < ' . CC_OWNER_MIN_LENGTH . ') {' . "\n" .
    '      error_message = error_message + "' . MODULE_PAYMENT_QUANTUMGATEWAY_TEXT_JS_CC_OWNER . '";' . "\n" .
             '      error = 1;' . "\n" .
             '    }' . "\n" .
             '    if (cc_number == "" || cc_number.length < ' . CC_NUMBER_MIN_LENGTH . ') {' . "\n" .
             '      error_message = error_message + "' . MODULE_PAYMENT_QUANTUMGATEWAY_TEXT_JS_CC_NUMBER . '";' . "\n" .
             '      error = 1;' . "\n" .
             '    }' . "\n";

      if (MODULE_PAYMENT_QUANTUMGATEWAY_COLLECT_CVV == 'True')  {
        $js .= '    if (cc_cvv == "" || cc_cvv.length < ' . CC_CVV_MIN_LENGTH . ') {' . "\n" .
      '      error_message = error_message + "' . MODULE_PAYMENT_QUANTUMGATEWAY_TEXT_JS_CC_CVV . '";' . "\n" .
               '      error = 1;' . "\n" .
               '    }' . "\n";
      }

      $js .= '  }' . "\n";
      return $js;
    } // </javascript_validation>


    /*
     * selection - builds set of input fields for collecting cc info
     */
    function selection() {
      global $order;

      for ($i=1; $i<13; $i++) {
        $expires_month[] = array('id' => sprintf('%02d', $i), 'text' => strftime('%B',mktime(0,0,0,$i,1,2000)));
      }

      $today = getdate(); 
      for ($i=$today['year']; $i < $today['year']+10; $i++) {
        $expires_year[] = array('id' => strftime('%y',mktime(0,0,0,1,1,$i)), 'text' => strftime('%Y',mktime(0,0,0,1,1,$i)));
      }

      // zen-cart is collecting the info, we must allow the user to enter it
      $selection = array('id' => $this->code,
                         'module' => $this->title,
                         'fields' => array(array('title' => MODULE_PAYMENT_QUANTUMGATEWAY_TEXT_CREDIT_CARD_OWNER,
                                                 'field' => zen_draw_input_field('cc_owner', $order->billing['firstname'] . ' ' . $order->billing['lastname'], 'id="' . $this->code . '-cc-owner"' . $onFocus),
						   'tag' => $this->code . '-cc-owner'),
                                           array('title' => MODULE_PAYMENT_QUANTUMGATEWAY_TEXT_CREDIT_CARD_NUMBER,
                                                 'field' => zen_draw_input_field('cc_number', '', 'id="' . $this->code . '-cc-number"' . $onFocus),
						   'tag' => $this->code . '-cc-number'),
                                           array('title' => MODULE_PAYMENT_QUANTUMGATEWAY_TEXT_CREDIT_CARD_EXPIRES,
                                                 'field' => zen_draw_pull_down_menu('cc_expires_month', $expires_month, '', 'id="' . $this->code . '-cc-expires-month"' . $onFocus) . '&nbsp;' . zen_draw_pull_down_menu('cc_expires_year', $expires_year, '', 'id="' . $this->code . '-cc-expires-year"' . $onFocus),
						   'tag' => $this->code . '-cc-expires-month')
			));

      if (MODULE_PAYMENT_QUANTUMGATEWAY_COLLECT_CVV == 'True') {
        $selection['fields'][] = array('title' => MODULE_PAYMENT_QUANTUMGATEWAY_TEXT_CREDIT_CARD_CVV,
                                       'field' => zen_draw_input_field('cc_cvv', '', 'size="4" maxlength="4" id="' . $this->code . '-cc-cvv"' . $onFocus),
                                         'tag' => $this->code . '-cc-cvv');
      }

      return $selection;
    } // </selection>


    /*
     * pre_confirmation_check - validates cc information
     */
    function pre_confirmation_check() {
      global $_POST, $messageStack;

      include(DIR_WS_CLASSES . 'cc_validation.php');

      $cc_validation = new cc_validation();
      $result = $cc_validation->validate($_POST['cc_number'], $_POST['cc_expires_month'], $_POST['cc_expires_year'], $_POST['cc_cvv']);

      $error = '';
      switch ($result) {
        case -1:
          $error = sprintf(TEXT_CCVAL_ERROR_UNKNOWN_CARD, substr($cc_validation->cc_number, 0, 4));
          break;
        case -2:
        case -3:
        case -4:
          $error = TEXT_CCVAL_ERROR_INVALID_DATE;
          break;
        case false:
          $error = TEXT_CCVAL_ERROR_INVALID_NUMBER;
          break;
      }

      if ( ($result == false) || ($result < 1) ) {
        $payment_error = 'payment_error=' . $this->code . '&error=' . urlencode($error);

        zen_redirect(zen_href_link(FILENAME_CHECKOUT_PAYMENT, $payment_error, 'SSL', true, false));
      }

      $this->cc_card_type = $cc_validation->cc_type;
      $this->cc_card_number = $cc_validation->cc_number;
      $this->cc_expiry_month = $cc_validation->cc_expiry_month;
      $this->cc_expiry_year = $cc_validation->cc_expiry_year;
    } // </pre_confirmation_check>


    /*
     * confirmation - display credit card information
     */
    function confirmation() {
      global $_POST;

      $confirmation = array('title' => $this->title . ': ' . $this->cc_card_type,
                            'fields' => array(array('title' => MODULE_PAYMENT_QUANTUMGATEWAY_TEXT_CREDIT_CARD_OWNER,
                            'field' => $_POST['cc_owner']),
                      array('title' => MODULE_PAYMENT_QUANTUMGATEWAY_TEXT_CREDIT_CARD_NUMBER,
                            'field' => substr($this->cc_card_number, 0, 4) . str_repeat('X', (strlen($this->cc_card_number) - 8)) . substr($this->cc_card_number, -4)),
                      array('title' => MODULE_PAYMENT_QUANTUMGATEWAY_TEXT_CREDIT_CARD_EXPIRES,
                            'field' => strftime('%B, %Y', mktime(0,0,0,$_POST['cc_expires_month'], 1, '20' . $_POST['cc_expires_year'])))));

      if (MODULE_PAYMENT_QUANTUMGATEWAY_COLLECT_CVV == 'True') {
	$confirmation['fields'][] = array('title' => MODULE_PAYMENT_QUANTUMGATEWAY_TEXT_CREDIT_CARD_CVV,
					  'field' => $_POST['cc_cvv']);
      }
      return $confirmation;
    } // </confirmation>


    /*
     * process_button - build data and actions to process
     */
    function process_button() {
      global $_POST, $order;
        $payment_error = 'payment_error=' . $this->code . '&error=' . urlencode(MODULE_PAYMENT_QUANTUMGATEWAY_TEXT_ERROR_MESSAGE);

      $approved_url = zen_href_link(FILENAME_CHECKOUT_PROCESS, 'referer=' . $this->code, 'SSL');
      $declined_url = zen_href_link(FILENAME_CHECKOUT_PAYMENT, 'error_message=' . urlencode(MODULE_PAYMENT_QUANTUMGATEWAY_TEXT_ERROR_MESSAGE), 'SSL', true, false);
      //$declined_url = zen_redirect(zen_href_link(FILENAME_CHECKOUT_PAYMENT, $payment_error, 'SSL', true, false));

      // Session control parameters
      $process_button_string = 
	   zen_draw_hidden_field('ID', $_REQUEST['zenid']) .
	   zen_draw_hidden_field('agent', MODULE_PAYMENT_QUANTUMGATEWAY_AGENT) .
	   zen_draw_hidden_field('approved', $approved_url) .
	   zen_draw_hidden_field('declined', $declined_url) .

      // QuantumGateway access parameters
      $process_button_string .= 
           zen_draw_hidden_field('gwlogin', MODULE_PAYMENT_QUANTUMGATEWAY_LOGIN) .
           zen_draw_hidden_field('RestrictKey', MODULE_PAYMENT_QUANTUMGATEWAY_RESTRICTKEY) .
           zen_draw_hidden_field('override_email_customer', ((MODULE_PAYMENT_QUANTUMGATEWAY_EMAIL_CUSTOMER == 'True') ? 'Y' : 'N')) .
           zen_draw_hidden_field('override_trans_email', ((MODULE_PAYMENT_QUANTUMGATEWAY_TRANS_EMAIL == 'True') ? 'Y' : 'N')) .
            zen_draw_hidden_field('MAXMIND', ((MODULE_PAYMENT_QUANTUMGATEWAY_MAXMIND == 'True') ? '1': '2'));

      // Credit card processing parameters
      $process_button_string .= 
            zen_draw_hidden_field('trans_method', 'CC') .
            zen_draw_hidden_field('ccnum', $this->cc_card_number) .
            zen_draw_hidden_field('ccmo', $this->cc_expiry_month) .
            zen_draw_hidden_field('ccyr', substr($this->cc_expiry_year, -2));

      if (MODULE_PAYMENT_QUANTUMGATEWAY_COLLECT_CVV == 'True') {
        $process_button_string .= zen_draw_hidden_field('CVVtype', '1') .
        $process_button_string .= zen_draw_hidden_field('CVV2', $_POST['cc_cvv']);
      }
      else {
        $process_button_string .= zen_draw_hidden_field('CVVtype', '0');
      }

      // Transaction Details
      $process_button_string .= 
zen_draw_hidden_field('amount', number_format($order->info['total'], 2)) .
zen_draw_hidden_field('FNAME', $order->billing['firstname']) .
zen_draw_hidden_field('LNAME', $order->billing['lastname']) .
zen_draw_hidden_field('BADDR1', $order->billing['street_address']) .
zen_draw_hidden_field('BCITY', $order->billing['city']) .
zen_draw_hidden_field('BSTATE', $order->billing['state']) .
zen_draw_hidden_field('BZIP1', $order->billing['postcode']) .
zen_draw_hidden_field('BCOUNTRY', $order->billing['country']['title']) .
zen_draw_hidden_field('PHONE', $order->customer['telephone']) .
zen_draw_hidden_field('BCUST_EMAIL', $order->customer['email_address']);

      return $process_button_string;
    } // </process_button>


    /*
     * before_process - store credit card info to the order
     */
    function before_process() {
        global $_POST, $order;

	$order->info['cc_type'] = $_POST['cc_type'];
	$order->info['cc_owner'] = $_POST['cc_owner'];
	$order->info['cc_expires'] = $_POST['cc_expires'];
	$order->info['cc_cvv'] = $_POST['cc_cvv'];
    } // </before_process>


    /*
     * after_process - send collected information
     */
    function after_process() {
      return false;
    } // </after_process>


    /*
     * get_error - display error details
     */
    function get_error() {
      global $_GET;

      $error = array('title' => MODULE_PAYMENT_QUANTUMGATEWAY_TEXT_ERROR,
                     'error' => stripslashes(urldecode($_GET['error'])));

      return $error;
    } // </get_error>


    /*
     * check - see whether module is installed
     */
    function check() {
      global $db;
      if (!isset($this->_check)) {
        $check_query = $db->Execute("select configuration_value from " . TABLE_CONFIGURATION . " where configuration_key = 'MODULE_PAYMENT_QUANTUMGATEWAY_STATUS'");
        $this->_check = $check_query->RecordCount();
      }
      return $this->_check;
    } // </check>


    /*
     * install payment module and its configuration settings
     */
    function install() {
      global $db;

      $db->Execute("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, set_function, date_added) values ('Enable QuantumGateway Module', 'MODULE_PAYMENT_QUANTUMGATEWAY_STATUS', 'True', 'Do you want to accept QuantumGateway payments?', '6', '0', 'zen_cfg_select_option(array(\'True\', \'False\'), ', now())");

      $db->Execute("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, set_function, use_function, date_added) values ('Set Order Status', 'MODULE_PAYMENT_QUANTUMGATEWAY_ORDER_STATUS_ID', '2', 'Set the status of orders made with this payment module to this value<br  />(\'Processing\' recommended)', '6', '6', 'zen_cfg_pull_down_order_statuses(', 'zen_get_order_status_name', now())");

      $db->Execute("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, date_added) values ('Sort order of display.', 'MODULE_PAYMENT_QUANTUMGATEWAY_SORT_ORDER', '0', 'Sort order of display. Lowest is displayed first.', '6', '134' , now())");

      $db->Execute("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, date_added) values ('Login Username', 'MODULE_PAYMENT_QUANTUMGATEWAY_LOGIN', 'testing', 'The login username used for the QuantumGateway service', '6', '2', now())");

$db->Execute("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, date_added) values ('Restriction Key', 'MODULE_PAYMENT_QUANTUMGATEWAY_RESTRICTKEY', '', 'Restriction Key used for restricting processing without key.  Note: You must have this enabled in your Quantumgateway Processing Config.', '6', '3', now())");

      $db->Execute("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, set_function, date_added) values ('Customer Notifications', 'MODULE_PAYMENT_QUANTUMGATEWAY_EMAIL_CUSTOMER', 'True', 'Should Quatnumgateway e-mail a receipt to the customer?', '6', '3', 'zen_cfg_select_option(array(\'True\', \'False\'), ', now())");

      $db->Execute("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, set_function, date_added) values ('Transaction Notifications', 'MODULE_PAYMENT_QUANTUMGATEWAY_TRANS_EMAIL', 'False', 'Should Quatnumgateway send transaction e-mail?', '6', '3', 'zen_cfg_select_option(array(\'True\', \'False\'), ', now())");

      $db->Execute("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, set_function, date_added) values ('Collect CVV information', 'MODULE_PAYMENT_QUANTUMGATEWAY_COLLECT_CVV', 'False', 'Enable?', '6', '4', 'zen_cfg_select_option(array(\'True\', \'False\'), ', now())");

      $db->Execute("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, set_function, date_added) values ('Maxmind Fraud Control', 'MODULE_PAYMENT_QUANTUMGATEWAY_MAXMIND', 'False', 'Enable?', '6', '4', 'zen_cfg_select_option(array(\'True\', \'False\'), ', now())");

    } // </install>


    /*
     * remove module and all its settings
     */
    function remove() {
      global $db;
      $db->Execute("delete from " . TABLE_CONFIGURATION . " where configuration_key in ('" . implode("', '", $this->keys()) . "')");
    } // </remove>


    /*
     * keys - internal list of configuration keys
     */
    function keys() {
      return array('MODULE_PAYMENT_QUANTUMGATEWAY_STATUS', 
	  'MODULE_PAYMENT_QUANTUMGATEWAY_ORDER_STATUS_ID',
	  'MODULE_PAYMENT_QUANTUMGATEWAY_SORT_ORDER',
	  'MODULE_PAYMENT_QUANTUMGATEWAY_LOGIN',
	  'MODULE_PAYMENT_QUANTUMGATEWAY_RESTRICTKEY',
	  'MODULE_PAYMENT_QUANTUMGATEWAY_EMAIL_CUSTOMER',
	  'MODULE_PAYMENT_QUANTUMGATEWAY_TRANS_EMAIL',
	  'MODULE_PAYMENT_QUANTUMGATEWAY_COLLECT_CVV',
	  'MODULE_PAYMENT_QUANTUMGATEWAY_MAXMIND'
      );
    } // </keys>
  }
?>
